@@ -20,7 +20,8 @@ import struct
 
 from mutagen import StreamInfo
 from mutagen._vorbis import VCommentDict
-from mutagen._util import cdata
+from mutagen._util import cdata, get_size, loadfile, convert_error
+from mutagen._tags import PaddingInfo
 from mutagen.ogg import OggPage, OggFileType, error as OggError
 
 
@@ -33,15 +34,19 @@ class OggTheoraHeaderError(error):
 
 
 class OggTheoraInfo(StreamInfo):
-    """Ogg Theora stream information.
+    """OggTheoraInfo()
+
+    Ogg Theora stream information.
 
     Attributes:
-
-    * length - file length in seconds, as a float
-    * fps - video frames per second, as a float
+        length (`float`): File length in seconds, as a float
+        fps (`float`): Video frames per second, as a float
+        bitrate (`int`): Bitrate in bps (int)
     """
 
     length = 0
+    fps = 0
+    bitrate = 0
 
     def __init__(self, fileobj):
         page = OggPage(fileobj)
@@ -63,13 +68,16 @@ class OggTheoraInfo(StreamInfo):
 
     def _post_tags(self, fileobj):
         page = OggPage.find_last(fileobj, self.serial)
+        if page is None:
+            raise OggTheoraHeaderError
         position = page.position
         mask = (1 << self.granule_shift) - 1
         frames = (position >> self.granule_shift) + (position & mask)
         self.length = frames / float(self.fps)
 
     def pprint(self):
-        return "Ogg Theora, %.2f seconds, %d bps" % (self.length, self.bitrate)
+        return u"Ogg Theora, %.2f seconds, %d bps" % (self.length,
+                                                      self.bitrate)
 
 
 class OggTheoraCommentDict(VCommentDict):
@@ -84,9 +92,10 @@ class OggTheoraCommentDict(VCommentDict):
                 pages.append(page)
                 complete = page.complete or (len(page.packets) > 1)
         data = OggPage.to_packets(pages)[0][7:]
-        super(OggTheoraCommentDict, self).__init__(data + b"\x01")
+        super(OggTheoraCommentDict, self).__init__(data, framing=False)
+        self._padding = len(data) - self._size
 
-    def _inject(self, fileobj):
+    def _inject(self, fileobj, padding_func):
         """Write tag data into the Theora comment packet/page."""
 
         fileobj.seek(0)
@@ -102,20 +111,40 @@ class OggTheoraCommentDict(VCommentDict):
 
         packets = OggPage.to_packets(old_pages, strict=False)
 
-        packets[0] = b"\x81theora" + self.write(framing=False)
+        content_size = get_size(fileobj) - len(packets[0])  # approx
+        vcomment_data = b"\x81theora" + self.write(framing=False)
+        padding_left = len(packets[0]) - len(vcomment_data)
 
-        new_pages = OggPage.from_packets(packets, old_pages[0].sequence)
+        info = PaddingInfo(padding_left, content_size)
+        new_padding = info._get_padding(padding_func)
+
+        packets[0] = vcomment_data + b"\x00" * new_padding
+
+        new_pages = OggPage._from_packets_try_preserve(packets, old_pages)
         OggPage.replace(fileobj, old_pages, new_pages)
 
 
 class OggTheora(OggFileType):
-    """An Ogg Theora file."""
+    """OggTheora(filething)
+
+    An Ogg Theora file.
+
+    Arguments:
+        filething (filething)
+
+    Attributes:
+        info (`OggTheoraInfo`)
+        tags (`mutagen._vorbis.VCommentDict`)
+    """
 
     _Info = OggTheoraInfo
     _Tags = OggTheoraCommentDict
     _Error = OggTheoraHeaderError
     _mimes = ["video/x-theora"]
 
+    info = None
+    tags = None
+
     @staticmethod
     def score(filename, fileobj, header):
         return (header.startswith(b"OggS") *
@@ -125,7 +154,19 @@ class OggTheora(OggFileType):
 Open = OggTheora
 
 
-def delete(filename):
-    """Remove tags from a file."""
+@convert_error(IOError, error)
+@loadfile(method=False, writable=True)
+def delete(filething):
+    """ delete(filething)
 
-    OggTheora(filename).delete()
+    Arguments:
+        filething (filething)
+    Raises:
+        mutagen.MutagenError
+
+    Remove tags from a file.
+    """
+
+    t = OggTheora(filething)
+    filething.fileobj.seek(0)
+    t.delete(filething)
